@@ -20,6 +20,23 @@ inode of the log file. This could be very useful if a rotate mechanism moves
 and zip the log file but you shouldn't forget that inodes aren't available on
 windows.
 
+=head1 CHANGES FROM 0.11
+
+The log level numbers changed because they wasn't in the right order!
+
+    8 nothing   is still the same
+    7 emergency is now 0
+    6 alert     is now 1
+    5 crit      is now 2
+    4 error     is now 3
+    3 warning   is now 4
+    2 notice    is now 5
+    1 info      is now 6
+    0 debug     is now 7
+
+If you set C<minlevel> and C<maxlevel> as strings in earlier versions than you don't need
+to change your code, but if you used numbers than you have to change it!
+
 =head1 METHODS
 
 =head2 new()
@@ -29,27 +46,57 @@ Call C<new()> to create a new log handler object.
 The C<new()> method expected the options for the log file. The only one mandatory
 option is C<filename>. All other options will be set to a default value.
 
+=head2 set_prefix()
+
+Call C<set_prefix()> to modifier the option prefix after you called C<new()>.
+
+    my $log = Log::Handler->new(
+       filename => 'file.log',
+       mode => 'append',
+       prefix => "myhost:$$ [<--LEVEL-->] "
+    );
+
+    $log->set_prefix("[<--LEVEL-->] myhost:$$ ");
+
+=head2 get_prefix()
+
+Call C<get_prefix()> to get the current prefix if you just want to modifier the current
+prefix and re-set the old.
+
+    # safe the old prefix
+    my $old_prefix = $log->get_prefix();
+
+    # set a new one for a code part in your script
+    $log->set_prefix("my new prefix");
+
+    # now set the your old prefix again
+    $log->set_prefix($old_prefix);
+
+Or you want to add something to the current prefix:
+
+    $log->set_prefix($log->get_prefix."add something");
+
 =head2 Log levels
 
 There are eigth log level and twelve methods to handle this levels:
 
 =over 4
 
-=item debug()
-
-=item info()
-
-=item notice(), note()
-
-=item warning()
-
-=item error(), err()
-
-=item critical(), crit()
+=item emergency(), emerg()
 
 =item alert()
 
-=item emergency(), emerg()
+=item critical(), crit()
+
+=item error(), err()
+
+=item warning()
+
+=item notice(), note()
+
+=item info()
+
+=item debug()
 
 =back
 
@@ -79,21 +126,21 @@ Both calls would log (provided that the log level INFO would log)
 
 =over 4
 
-=item would_log_debug()
-
-=item would_log_info()
-
-=item would_log_notice(), would_log_note()
-
-=item would_log_warning()
-
-=item would_log_error(), would_log_err()
-
-=item would_log_critical(), would_log_crit()
+=item would_log_emergency(), would_log_emerg()
 
 =item would_log_alert()
 
-=item would_log_emergency(), would_log_emerg()
+=item would_log_critical(), would_log_crit()
+
+=item would_log_error(), would_log_err()
+
+=item would_log_warning()
+
+=item would_log_notice(), would_log_note()
+
+=item would_log_info()
+
+=item would_log_debug()
 
 =back
 
@@ -107,7 +154,7 @@ pass the dump in any case, because it would costs a lot of resources.
     $log->debug(Dumper($hash));
 
 This example would dump the $hash in any case and handoff it to the log handler, 
-but this isn't what we really want!
+but this isn't that what we really want!
 
     $log->debug(Dumper($hash))
        if $log->would_log_debug();
@@ -117,11 +164,14 @@ Now we dump the $hash only if the current log level would really log it.
 The methods C<would_log_note()>, C<would_log_err()>, C<would_log_crit()> and
 C<would_log_emerg()> are just shortcuts.
 
+All methods that would NOT log the message returns TRUE.
+
 =head2 errstr()
 
 Call C<errstr()> if you want to get the last error string. This is useful with the
 option C<die_on_errors>. If you set this option to 0 the handler wouldn't croak
-on errors. Set C<die_on_errors> to control it yourself.
+if simple write operations failed. Set C<die_on_errors> to control it yourself.
+C<errstr()> is only useful with C<new()>, C<CLOSE()> and the log level methods.
 
     $log->info("log information") or die $log->errstr;
 
@@ -139,8 +189,8 @@ because on missing params or on wrong settings!
 
 This would croaks, because option C<mode> except C<append> or C<trunc> or C<excl>.
 
-If you set the option C<fileopen> to 1 to open the log file permanent and the call
-of C<new> failed then you can absorb the error.
+If you set the option C<fileopen> to 1 - the default - to open the log file permanent
+and the call of C<new> failed then you can absorb the error.
 
     my $log = Log::Handler->new(filename => 'file.log')
        or warn Log::Handler->errstr;
@@ -212,6 +262,23 @@ This option works only if option C<fileopen> is set to 1.
     0 - deactivate
     1 - try to reopen logfile if the inode changed (default)
 
+=head2 filename and reopen
+
+Please note that it's better to set C<reopen> and C<filename> to 0 on Windows systems
+because Windows unfortunately haven't the faintest idea of inodes.
+
+To write your code independent you should control that:
+
+    my $os_is_win = $^O =~ /win/i ? 0 : 1;
+
+    my $log = Log::Handler->new(
+       filename => 'file.log',
+       mode => 'append',
+       fileopen => $os_is_win
+    );
+
+To set C<filename> to 0 implies that C<reopen> has no importance.
+
 =head2 mode
 
 There are three possible modes to open a log file.
@@ -298,24 +365,24 @@ Take a look to the EXAMPLES to see more.
 With the options C<maxlevel> and C<minlevel> you can set the log levels you
 wish to log to your log file. The log levels are:
 
-    0 - debug
-    1 - info
-    2 - notice, note
-    3 - warning
-    4 - error, err
-    5 - critical, crit
-    6 - alert
-    7 - emergency, emerg
+    7 - debug
+    6 - info
+    5 - notice, note
+    4 - warning
+    3 - error, err
+    2 - critical, crit
+    1 - alert
+    0 - emergency, emerg
 
 The levels C<note>, C<err>, C<crit> and C<emerg> are just shortcuts.
 
 It's possible to set the log level as a string or as number. The default
-C<maxlevel> is 3 and the default C<minlevel> is 7.
+C<maxlevel> is 4 and the default C<minlevel> is 0.
 
-Example: If C<maxlevel> is set to 4 and C<minlevel> to 7 then only emergency (emerg),
-alert, critical (crit) and error (err) messages will be logged to the logfile.
+Example: If C<maxlevel> is set to 4 and C<minlevel> to 0 then only emergency (emerg),
+alert, critical (crit) and error (err) messages will be logged to the log file.
 
-You can set both to 8 if you don't want to log any message.
+You can set both to 8 or "nothing" if you don't want to log any message.
 
 =head2 die_on_errors
 
@@ -338,8 +405,8 @@ because on missing params or wrong settings.
        filename => 'file1.log',
        mode     => 'append',
        newline  => 1,
-       maxlevel => 0,
-       minlevel => 7
+       maxlevel => 7,
+       minlevel => 0
     );
 
     $log->debug("this is a debug message");
@@ -377,8 +444,8 @@ Would log this:
     my $log = Log::Handler->new(
        filename   => '/var/run/pid-file1',
        mode       => 'trunc',
-       maxlevel   => 2,
-       minlevel   => 2,
+       maxlevel   => 5,
+       minlevel   => 5,
        prefix     => '',
        timeformat => ''
     );
@@ -400,7 +467,7 @@ Would truncate /var/run/pid-file1 and write just the pid to the logfile.
     my $log = Log::Handler->new(
        filename => "${progname}.log",
        mode     => 'append',
-       maxlevel => 1,
+       maxlevel => 6,
        newline  => 1,
        prefix   => "${hostname}[$pid] [<--LEVEL-->] $progname: "
     );
@@ -421,7 +488,7 @@ Would log:
     my $log = Log::Handler->new(
        filename   => 'file1.log',
        mode       => 'trunc',
-       maxlevel   => 3,
+       maxlevel   => 4,
        prefix     => '',
        timeformat => ''
     );
@@ -509,7 +576,7 @@ SUCH DAMAGES.
 
 package Log::Handler;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 use strict;
 use warnings;
@@ -520,23 +587,20 @@ use POSIX qw(strftime);
 use Params::Validate;
 use Carp qw(croak);
 
-use constant DEBUG     =>  0;
-use constant INFO      =>  1;
-use constant NOTICE    =>  2;
-use constant NOTE      =>  2;
-use constant WARNING   =>  3;
-use constant ERROR     =>  4;
-use constant ERR       =>  4;
-use constant CRITICAL  =>  5;
-use constant CRIT      =>  5;
-use constant ALERT     =>  6;
-use constant EMERGENCY =>  7;
-use constant EMERG     =>  7;
+use constant EMERGENCY =>  0;
+use constant EMERG     =>  0;
+use constant ALERT     =>  1;
+use constant CRITICAL  =>  2;
+use constant CRIT      =>  2;
+use constant ERROR     =>  3;
+use constant ERR       =>  3;
+use constant WARNING   =>  4;
+use constant NOTICE    =>  5;
+use constant NOTE      =>  5;
+use constant INFO      =>  6;
+use constant DEBUG     =>  7;
 use constant NOTHING   =>  8;
 
-# global place holder for error strings
-$__PACKAGE__::errstr = "";
-
 # The BEGIN block is used to generate the syslog methods
 # and the would_log_* methods. The syslog methods calls
 # _print() with the syslog level as first argument. The
@@ -561,8 +625,8 @@ BEGIN {
          *{"would_log_$routine"} = sub {
             my $self = shift;
             return 1
-               if &{$level} >= $self->{maxlevel}
-               && &{$level} <= $self->{minlevel};
+               if &{$level} >= $self->{minlevel}
+               && &{$level} <= $self->{maxlevel};
             return undef;
          };
       } # end "no strict" block
@@ -573,21 +637,25 @@ sub new {
    my $class = shift;
    my %opts  = ();
    my $self  = bless \%opts, $class;
+   my $bool  = qr/^[10]$/;
 
    %opts = Params::Validate::validate(@_, {
       filename => {
          type => Params::Validate::SCALAR | Params::Validate::GLOBREF,
       },
       filelock => {
-         type => Params::Validate::BOOLEAN,
+         type => Params::Validate::SCALAR,
+         regex => $bool,
          default => 1,
       },
       fileopen => {
-         type => Params::Validate::BOOLEAN,
+         type => Params::Validate::SCALAR,
+         regex => $bool,
          default => 1,
       },
       reopen => {
-         type  => Params::Validate::BOOLEAN,
+         type  => Params::Validate::SCALAR,
+         regex => $bool,
          default => 1,
       },
       mode => {
@@ -596,7 +664,8 @@ sub new {
          default => 'excl',
       },
       autoflush => {
-         type => Params::Validate::BOOLEAN,
+         type => Params::Validate::SCALAR,
+         regex => $bool,
          default => 1,
       },
       permissions => {
@@ -609,25 +678,27 @@ sub new {
          default => '%b %d %H:%M:%S',
       },
       newline => {
-         type => Params::Validate::BOOLEAN,
+         type => Params::Validate::SCALAR,
+         regex => $bool,
          default => 0,
       },
       prefix => {
          type => Params::Validate::SCALAR,
          default => '[<--LEVEL-->] ',
       },
-      maxlevel => {
-         type => Params::Validate::SCALAR,
-         regex => qr/^([0-8]|debug|info|notice|note|warning|error|err|critical|crit|alert|emergency|emerg)$/,
-         default => 3,
-      },
       minlevel => {
          type => Params::Validate::SCALAR,
-         regex => qr/^([0-8]|debug|info|notice|note|warning|error|err|critical|crit|alert|emergency|emerg)$/,
-         default => 7,
+         regex => qr/^([0-8]|nothing|debug|info|notice|note|warning|error|err|critical|crit|alert|emergency|emerg)$/,
+         default => 0,
+      },
+      maxlevel => {
+         type => Params::Validate::SCALAR,
+         regex => qr/^([0-8]|nothing|debug|info|notice|note|warning|error|err|critical|crit|alert|emergency|emerg)$/,
+         default => 4,
       },
       die_on_errors => {
-         type => Params::Validate::BOOLEAN,
+         type => Params::Validate::SCALAR,
+         regex => $bool,
          default => 1,
       },
    });
@@ -677,6 +748,16 @@ sub new {
    return $self;
 }
 
+sub get_prefix {
+   my $self = shift;
+   return $self->{prefix};
+}
+
+sub set_prefix {
+   my $self = shift;
+   $self->{prefix} = shift;
+}
+
 sub CLOSE {
    my $self = shift;
 
@@ -688,7 +769,10 @@ sub CLOSE {
    return 1;
 }
 
-sub errstr { return $__PACKAGE__::errstr }
+sub errstr {
+   my $self = shift;
+   return $self->{errstr};
+}
 
 sub DESTROY {
    my $self = shift;
@@ -789,8 +873,8 @@ sub _print {
    {  # return if we don't want log this level
       no strict 'refs';
       return 1
-         unless &{$level} >= $self->{maxlevel}
-             && &{$level} <= $self->{minlevel};
+         unless &{$level} >= $self->{minlevel}
+             && &{$level} <= $self->{maxlevel};
    }
 
    if ($self->{fileopen} == 0) {
@@ -855,12 +939,11 @@ sub _gettime {
 }
 
 sub _raise_error {
-   my $self   = shift;
-   my $class  = ref($self);
-   my $errstr = shift;
-   croak "$class: $errstr"
+   my $self  = shift;
+   my $class = ref($self);
+   $self->{errstr} = defined $_[0] ? $_[0] : '';
+   croak "$class: $self->{errstr}"
       if $self->{die_on_errors} == 1;
-   $__PACKAGE__::errstr = $errstr;
    return undef;
 }
 
